@@ -6,6 +6,7 @@ import logging
 import celery
 import web3
 from cic_registry import CICRegistry
+from cic_registry.chain import ChainSpec
 
 # local imports
 from cic_eth.db import SessionBase
@@ -31,18 +32,6 @@ contract_function_signatures = {
 
 class BancorTxFactory(TxFactory):
 
-    local_inited = False
-
-    @staticmethod
-    def init(w3, chain_spec): #, from_address):
-        TxFactory.init(w3, chain_spec) # , from_address)
-        if BancorTxFactory.local_inited:
-            logg.debug('bancortxfactory already inited, skipping')
-            return
-        BancorTxFactory.network_contract = CICRegistry.get_contract(chain_spec, 'BancorNetwork')
-        BancorTxFactory.local_inited = True
-
-
     """Factory for creating Bancor network transactions.
     """
     def convert(
@@ -52,6 +41,7 @@ class BancorTxFactory(TxFactory):
             reserve_address,
             source_amount,
             minimum_return,
+            chain_spec,
             fee_beneficiary='0x0000000000000000000000000000000000000000',
             fee_ppm=0,
             ):
@@ -70,8 +60,9 @@ class BancorTxFactory(TxFactory):
         :return: Unsigned "convert" transaction in standard Ethereum format
         :rtype: dict
         """
-        network_gas = self.network_contract.gas('convert')
-        tx_convert_buildable = self.network_contract.contract.functions.convert2(
+        network_contract = CICRegistry.get_contract(chain_spec, 'BancorNetwork')
+        network_gas = network_contract.gas('convert')
+        tx_convert_buildable = network_contract.contract.functions.convert2(
             [
                 source_token_address,
                 source_token_address,
@@ -88,7 +79,7 @@ class BancorTxFactory(TxFactory):
                 'from': self.address,
                 'gas': network_gas,
                 'gasPrice': self.gas_price,
-                'chainId': BancorTxFactory.chain_spec.chain_id(),
+                'chainId': chain_spec.chain_id(),
                 'nonce': self.next_nonce(),
                 })
         return tx_convert
@@ -141,7 +132,7 @@ def unpack_convert(data):
 
 
 @celery_app.task(bind=True)
-def convert_with_default_reserve(self, tokens, from_address, source_amount, minimum_return, to_address=None):
+def convert_with_default_reserve(self, tokens, from_address, source_amount, minimum_return, to_address, chain_str):
     """Performs a conversion between two liquid tokens using Bancor network.
 
     :param tokens: Token pair, source and destination respectively
@@ -154,45 +145,44 @@ def convert_with_default_reserve(self, tokens, from_address, source_amount, mini
     :type minimum_return: int
     """
 
+    chain_spec = ChainSpec.from_chain_str(chain_str)
     queue = self.request.delivery_info['routing_key']
 
-    c = RpcClient(from_address)
-
-    chain_spec = BancorTxFactory.chain_spec
+    c = RpcClient(chain_spec, holder_address=from_address)
 
     cr = CICRegistry.get_contract(chain_spec, 'BancorNetwork')
     source_token = CICRegistry.get_address(chain_spec, tokens[0]['address'])
-    reserve_address = CICRegistry.get_contract(chain_spec, 'BNTToken').address()
+    reserve_address = CICRegistry.get_contract(chain_spec, 'BNTToken', 'ERC20').address()
 
-    TxFactory.init(c.w3, chain_spec) #, from_address)
-    tx_factory = TokenTxFactory(from_address)
-    
-    tx_approve_zero = tx_factory.approve(source_token.address(), cr.address(), 0)
-    (tx_approve_zero_hash_hex, tx_approve_zero_signed_hex) = sign_and_register_tx(tx_approve_zero, queue, 'cic_eth.eth.token.otx_cache_approve') 
+    tx_factory = TokenTxFactory(from_address, c)
+   
+    tx_approve_zero = tx_factory.approve(source_token.address(), cr.address(), 0, chain_spec)
+    (tx_approve_zero_hash_hex, tx_approve_zero_signed_hex) = sign_and_register_tx(tx_approve_zero, chain_str, queue, 'cic_eth.eth.token.otx_cache_approve') 
 
-    tx_approve = tx_factory.approve(source_token.address(), cr.address(), source_amount)
-    (tx_approve_hash_hex, tx_approve_signed_hex) = sign_and_register_tx(tx_approve, queue, 'cic_eth.eth.token.otx_cache_approve') 
+    tx_approve = tx_factory.approve(source_token.address(), cr.address(), source_amount, chain_spec)
+    (tx_approve_hash_hex, tx_approve_signed_hex) = sign_and_register_tx(tx_approve, chain_str, queue, 'cic_eth.eth.token.otx_cache_approve') 
 
-    BancorTxFactory.init(c.w3, chain_spec) #, from_address)
-    tx_factory = BancorTxFactory(from_address)
+    tx_factory = BancorTxFactory(from_address, c)
     tx_convert = tx_factory.convert(
             tokens[0]['address'],
             tokens[1]['address'],
             reserve_address,
             source_amount,
             minimum_return,
+            chain_spec,
             )
-    (tx_convert_hash_hex, tx_convert_signed_hex) = sign_and_register_tx(tx_convert, queue, 'cic_eth.eth.bancor.otx_cache_convert')
+    (tx_convert_hash_hex, tx_convert_signed_hex) = sign_and_register_tx(tx_convert, chain_str, queue, 'cic_eth.eth.bancor.otx_cache_convert')
 
     if to_address != None:
-        save_convert_recipient(tx_convert_hash_hex, to_address)
+        save_convert_recipient(tx_convert_hash_hex, to_address, chain_str)
 
     s = create_check_gas_and_send_task(
             [tx_approve_zero_signed_hex, tx_approve_signed_hex, tx_convert_signed_hex],
+            chain_str,
             from_address,
             tx_approve_zero['gasPrice'] * tx_approve_zero['gas'],
-            [tx_approve_hash_hex],
-            queue,
+            tx_hashes_hex=[tx_approve_hash_hex],
+            queue=queue,
             )
     s.apply_async()
     return tx_convert_hash_hex
@@ -242,7 +232,7 @@ def convert_with_default_reserve(self, tokens, from_address, source_amount, mini
 
 
 @celery_app.task()
-def save_convert_recipient(convert_hash, recipient_address):
+def save_convert_recipient(convert_hash, recipient_address, chain_str):
     """Registers the recipient target for a convert-and-transfer operation.
 
     :param convert_hash: Transaction hash of convert operation
@@ -251,7 +241,7 @@ def save_convert_recipient(convert_hash, recipient_address):
     :type recipient_address: str, 0x-hex
     """
     session = SessionBase.create_session()
-    t = TxConvertTransfer(convert_hash, recipient_address)
+    t = TxConvertTransfer(convert_hash, recipient_address, chain_str)
     session.add(t)
     session.commit()
     session.close()
@@ -278,7 +268,7 @@ def save_convert_transfer(convert_hash, transfer_hash):
 
 
 @celery_app.task()
-def resolve_converters_by_tokens(tokens):
+def resolve_converters_by_tokens(tokens, chain_str):
     """Return converters for a list of tokens.
 
     :param tokens: Token addresses to look up
@@ -286,7 +276,7 @@ def resolve_converters_by_tokens(tokens):
     :return: Addresses of matching converters
     :rtype: list of str, 0x-hex
     """
-    chain_spec = BancorTxFactory.chain_spec
+    chain_spec = ChainSpec.from_chain_str(chain_str)
     for t in tokens:
         c = CICRegistry.get_contract(chain_spec, 'ConverterRegistry')
         fn = c.function('getConvertersByAnchors')
@@ -300,7 +290,7 @@ def resolve_converters_by_tokens(tokens):
 
 
 @celery_app.task(bind=True)
-def transfer_converted(self, tokens, holder_address, receiver_address, value, tx_convert_hash_hex):
+def transfer_converted(self, tokens, holder_address, receiver_address, value, tx_convert_hash_hex, chain_str):
     """Execute the ERC20 transfer of a convert-and-transfer operation.
 
     First argument is a list of tokens, to enable the task to be chained to the symbol to token address resolver function. However, it accepts only one token as argument.
@@ -321,28 +311,30 @@ def transfer_converted(self, tokens, holder_address, receiver_address, value, tx
     if len(tokens) != 1:
         raise TokenCountError
 
+    chain_spec = ChainSpec.from_chain_str(chain_str)
+
     queue = self.request.delivery_info['routing_key']
 
-    chain_spec = BancorTxFactory.chain_spec
-    c = RpcClient(holder_address)
+    c = RpcClient(chain_spec, holder_address=holder_address)
 
     # get transaction parameters
     gas_price = c.gas_price()
-    TokenTxFactory.init(c.w3, chain_spec) #, holder_address)
-    tx_factory = TokenTxFactory(holder_address)
+    tx_factory = TokenTxFactory(holder_address, c)
 
     token_address = tokens[0]['address']
     tx_transfer = tx_factory.transfer(
         token_address,
         receiver_address,
         value,
+        chain_spec,
             )
-    (tx_transfer_hash_hex, tx_transfer_signed_hex) = sign_and_register_tx(tx_transfer, queue, 'cic_eth.eth.token.otx_cache_transfer')
+    (tx_transfer_hash_hex, tx_transfer_signed_hex) = sign_and_register_tx(tx_transfer, chain_str, queue, 'cic_eth.eth.token.otx_cache_transfer')
 
     # send transaction
     logg.info('transfer converted token {} from {} to {} value {} {}'.format(token_address, holder_address, receiver_address, value, tx_transfer_signed_hex))
     s = create_check_gas_and_send_task(
             [tx_transfer_signed_hex],
+            chain_str,
             holder_address,
             tx_transfer['gasPrice'] * tx_transfer['gas'],
             None,
@@ -365,9 +357,10 @@ def transfer_converted(self, tokens, holder_address, receiver_address, value, tx
 def otx_cache_convert(
         tx_hash_hex,
         tx_signed_raw_hex,
+        chain_str,
         ):
 
-    chain_spec = BancorTxFactory.chain_spec
+    chain_spec = ChainSpec.from_chain_str(chain_str)
     tx_signed_raw_bytes = bytes.fromhex(tx_signed_raw_hex[2:])
     tx = unpack_signed_raw_tx(tx_signed_raw_bytes, chain_spec.chain_id())
     tx_data = unpack_convert(tx['data'])
